@@ -3,6 +3,19 @@
 
 #include <clutter/clutter-actor.h>
 #include <cogl/cogl.h>
+#include <cairo.h>
+
+/* XXX: Some of the private parts in this header expect the
+ * CoglFramebuffer typedef from Cogl, but its possible for this header
+ * to be included without the experimental Cogl api having been
+ * selected by defining COGL_ENABLE_EXPERIMENTAL_API or
+ * COGL_ENABLE_EXPERIMENTAL_2_0_API.
+ *
+ * We declare a  place holder type ourselves in this case...
+ */
+#ifndef COGL_ENABLE_EXPERIMENTAL_API
+typedef struct _CoglFramebuffer CoglFramebuffer;
+#endif
 
 G_BEGIN_DECLS
 
@@ -56,14 +69,17 @@ struct _ClutterStageWindowIface
                                                  gint                width,
                                                  gint                height);
   void              (* get_geometry)            (ClutterStageWindow *stage_window,
-                                                 ClutterGeometry    *geometry);
+                                                 cairo_rectangle_int_t *geometry);
 
   int               (* get_pending_swaps)       (ClutterStageWindow *stage_window);
 
-  void              (* add_redraw_clip)         (ClutterStageWindow *stage_window,
-                                                 ClutterGeometry    *stage_rectangle);
-  gboolean          (* has_redraw_clips)        (ClutterStageWindow *stage_window);
-  gboolean          (* ignoring_redraw_clips)   (ClutterStageWindow *stage_window);
+  void              (* add_redraw_clip)         (ClutterStageWindow    *stage_window,
+                                                 cairo_rectangle_int_t *stage_rectangle);
+  gboolean          (* has_redraw_clips)        (ClutterStageWindow    *stage_window);
+  gboolean          (* ignoring_redraw_clips)   (ClutterStageWindow    *stage_window);
+  gboolean          (* get_redraw_clip_bounds)  (ClutterStageWindow    *stage_window,
+                                                 cairo_rectangle_int_t *clip);
+
 
   void              (* set_accept_focus)        (ClutterStageWindow *stage_window,
                                                  gboolean            accept_focus);
@@ -71,6 +87,8 @@ struct _ClutterStageWindowIface
   void              (* redraw)                  (ClutterStageWindow *stage_window);
 
   CoglFramebuffer  *(* get_active_framebuffer)  (ClutterStageWindow *stage_window);
+
+  gboolean          (* can_clip_redraws)        (ClutterStageWindow *stage_window);
 };
 
 GType clutter_stage_window_get_type (void) G_GNUC_CONST;
@@ -97,21 +115,25 @@ void              _clutter_stage_window_resize                  (ClutterStageWin
                                                                  gint                width,
                                                                  gint                height);
 void              _clutter_stage_window_get_geometry            (ClutterStageWindow *window,
-                                                                 ClutterGeometry    *geometry);
+                                                                 cairo_rectangle_int_t *geometry);
 int               _clutter_stage_window_get_pending_swaps       (ClutterStageWindow *window);
 
-void              _clutter_stage_window_add_redraw_clip         (ClutterStageWindow *window,
-                                                                 ClutterGeometry    *stage_clip);
-gboolean          _clutter_stage_window_has_redraw_clips        (ClutterStageWindow *window);
-gboolean          _clutter_stage_window_ignoring_redraw_clips   (ClutterStageWindow *window);
+void              _clutter_stage_window_add_redraw_clip         (ClutterStageWindow    *window,
+                                                                 cairo_rectangle_int_t *stage_clip);
+gboolean          _clutter_stage_window_has_redraw_clips        (ClutterStageWindow    *window);
+gboolean          _clutter_stage_window_ignoring_redraw_clips   (ClutterStageWindow    *window);
+gboolean          _clutter_stage_window_get_redraw_clip_bounds  (ClutterStageWindow    *window,
+                                                                 cairo_rectangle_int_t *clip);
 
 void              _clutter_stage_window_set_accept_focus        (ClutterStageWindow *window,
-                                                           gboolean            accept_focus);
+                                                                 gboolean            accept_focus);
 
 void              _clutter_stage_window_redraw                  (ClutterStageWindow *window);
 
 CoglFramebuffer  *_clutter_stage_window_get_active_framebuffer  (ClutterStageWindow *window);
 
+gboolean          _clutter_stage_window_can_clip_redraws        (ClutterStageWindow *window);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_STAGE_WINDOW_H__ */
