@@ -42,9 +42,7 @@
 #include <math.h>
 
 #define CLUTTER_DISABLE_DEPRECATION_WARNINGS
-#include "deprecated/clutter-actor.h"
 
-#include "clutter-alpha.h"
 #include "clutter-behaviour.h"
 #include "clutter-behaviour-rotate.h"
 #include "clutter-debug.h"
@@ -52,6 +50,10 @@
 #include "clutter-main.h"
 #include "clutter-private.h"
 
+G_DEFINE_TYPE (ClutterBehaviourRotate,
+               clutter_behaviour_rotate,
+               CLUTTER_TYPE_BEHAVIOUR);
+
 struct _ClutterBehaviourRotatePrivate
 {
   gdouble angle_start;
@@ -65,6 +67,11 @@ struct _ClutterBehaviourRotatePrivate
   gint center_z;
 };
 
+#define CLUTTER_BEHAVIOUR_ROTATE_GET_PRIVATE(obj) \
+        (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
+         CLUTTER_TYPE_BEHAVIOUR_ROTATE, \
+         ClutterBehaviourRotatePrivate))
+
 enum
 {
   PROP_0,
@@ -82,10 +89,6 @@ enum
 
 static GParamSpec *obj_props[PROP_LAST];
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterBehaviourRotate,
-                            clutter_behaviour_rotate,
-                            CLUTTER_TYPE_BEHAVIOUR)
-
 typedef struct {
   gdouble angle;
 } RotateFrameClosure;
@@ -260,6 +263,8 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
   ClutterBehaviourClass *behaviour_class = CLUTTER_BEHAVIOUR_CLASS (klass);
   GParamSpec *pspec = NULL;
 
+  g_type_class_add_private (klass, sizeof (ClutterBehaviourRotatePrivate));
+
   gobject_class->set_property = clutter_behaviour_rotate_set_property;
   gobject_class->get_property = clutter_behaviour_rotate_get_property;
 
@@ -393,19 +398,19 @@ clutter_behaviour_rotate_class_init (ClutterBehaviourRotateClass *klass)
 }
 
 static void
-clutter_behaviour_rotate_init (ClutterBehaviourRotate *self)
+clutter_behaviour_rotate_init (ClutterBehaviourRotate *rotate)
 {
-  self->priv = clutter_behaviour_rotate_get_instance_private (self);
+  ClutterBehaviourRotatePrivate *priv;
 
-  self->priv->angle_start = 0.0;
-  self->priv->angle_end = 0.0;
+  rotate->priv = priv = CLUTTER_BEHAVIOUR_ROTATE_GET_PRIVATE (rotate);
 
-  self->priv->axis = CLUTTER_Z_AXIS;
-  self->priv->direction = CLUTTER_ROTATE_CW;
+  priv->angle_start = priv->angle_end = 0;
 
-  self->priv->center_x = 0;
-  self->priv->center_y = 0;
-  self->priv->center_z = 0;
+  priv->axis = CLUTTER_Z_AXIS;
+
+  priv->direction = CLUTTER_ROTATE_CW;
+
+  priv->center_x = priv->center_y = priv->center_z = 0;
 }
 
 /**
