@@ -135,6 +135,7 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/actor", actor_replace_child);
   TEST_CONFORM_SIMPLE ("/actor", actor_remove_child);
   TEST_CONFORM_SIMPLE ("/actor", actor_remove_all);
+  TEST_CONFORM_SIMPLE ("/actor", actor_container_signals);
   TEST_CONFORM_SIMPLE ("/actor", actor_destruction);
   TEST_CONFORM_SIMPLE ("/actor", actor_anchors);
   TEST_CONFORM_SIMPLE ("/actor", actor_picking);
@@ -148,15 +149,15 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/actor/iter", actor_iter_traverse_children);
   TEST_CONFORM_SIMPLE ("/actor/iter", actor_iter_traverse_remove);
 
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_initial_state);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_shown_not_parented);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_realized);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_realize_not_recursive);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_map_recursive);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_mapped);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_show_on_set_parent);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_clone_no_map);
-  TEST_CONFORM_SIMPLE ("/actor/invariants", test_contains);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_initial_state);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_shown_not_parented);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_realized);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_realize_not_recursive);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_map_recursive);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_mapped);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_show_on_set_parent);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", clone_no_map);
+  TEST_CONFORM_SIMPLE ("/actor/invariants", actor_contains);
   TEST_CONFORM_SIMPLE ("/actor/invariants", default_stage);
 
   TEST_CONFORM_SIMPLE ("/actor/opacity", test_label_opacity);
@@ -233,23 +234,15 @@ main (int argc, char **argv)
 
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_object);
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_fixed);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_backface_culling);
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_materials);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_pipeline_user_matrix);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_blend_strings);
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_premult);
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_readpixels);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_path);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_depth_test);
 
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_npot_texture);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_multitexture);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_mipmaps);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_sub_texture);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_pixel_array);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_rectangle);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_3d);
-  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_wrap_modes);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_pixmap_x11);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_get_set_data);
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_atlas_migration);
@@ -258,14 +251,9 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_interleved);
   TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_mutability);
 
-  TEST_CONFORM_SIMPLE ("/cogl/vertex-array", test_cogl_primitive);
-
-  TEST_CONFORM_SIMPLE ("/cogl/shaders", test_cogl_just_vertex_shader);
-
   /* left to the end because they aren't currently very orthogonal and tend to
    * break subsequent tests! */
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_viewport);
-  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_offscreen);
 
   return g_test_run ();
 }
